package com.jtxt.demo.controller;

import com.jtxt.demo.model.*;
import com.jtxt.demo.repository.ExamRepository;
import com.jtxt.demo.repository.ExamToQuestionRepository;
import com.jtxt.demo.repository.QuestionRepository;
import com.jtxt.demo.service.EventService;
import com.jtxt.demo.service.ExamService;
import com.jtxt.demo.service.PointsHistoryService;
import com.jtxt.demo.service.QuestionService;
import com.jtxt.demo.util.Utils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.io.UnsupportedEncodingException;
import java.time.LocalDateTime;
import java.time.temporal.WeekFields;
import java.util.List;
import java.util.Locale;

import static com.jtxt.demo.util.Utils.CHINA_ZONE;

@RestController
public class ExamController {
    @Autowired
    private ExamService examService;
    @Autowired
    private QuestionService questionService;
    @Autowired
    private PointsHistoryService pointsHistoryService;
    @Autowired
    private ExamRepository examRepository;
    @Autowired
    private QuestionRepository questionRepository;
    @Autowired
    private EventService eventService;
    @Autowired
    private ExamToQuestionRepository examToQuestionRepository;

    private static final String DAILY_QUESTIONS = "daily-questions";
    private static final String WEEKLY_QUESTIONS = "weekly-questions";
    private static final int DAILY_QUESTIONS_NUM = 5;
    private static final String DAILY_POINTS_DESCRIPTION = "通过每日答题所得";
    private static final String WEEKLY_POINTS_DESCRIPTION = "通过每周答题所得";

    @PostMapping("/admin/exams")
    public ResponseEntity<?> createExam(@RequestBody Exam exam) {
        Exam newExam = examService.createExam(exam);
        return new ResponseEntity<>(newExam, HttpStatus.OK);
    }
    @GetMapping("/exams/{examId}")
    public ResponseEntity<?> getExam(@PathVariable String examId) {

        Exam exam = examService.getExamById(examId);
        if (exam == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(exam, HttpStatus.OK);
    }
    @PutMapping("/exams/{examId}")
    public ResponseEntity<?> editExam(@PathVariable String examId, @RequestBody Exam updatedExam) {
        Exam exam = examService.getExamById(examId);
        if (exam == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        updatedExam.setId(examId);
        return new ResponseEntity<>(examService.editExam(updatedExam), HttpStatus.OK);
    }
    // Note this is dangerous
    @DeleteMapping("/admin/questions")
    public ResponseEntity<?> deleteAllQuestions() {
        questionRepository.deleteAll();
        examToQuestionRepository.deleteAll();
        return new ResponseEntity<>(HttpStatus.OK);
    }
    // Note this is dangerous
    @DeleteMapping("/admin/exams")
    public ResponseEntity<?> deleteAllExams() {
        examRepository.deleteAll();
        return new ResponseEntity<>(HttpStatus.OK);
    }
    @DeleteMapping("/exams/{examId}")
    public ResponseEntity<?> deleteExam(@PathVariable String examId) {
        Exam exam = examService.getExamById(examId);
        if (exam == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(examService.deleteExam(exam), HttpStatus.OK);
    }
    // 根据工种不同，只返回该工种的问题
    @PostMapping("/exams/{examId}/start")
    public ResponseEntity<?> startExam(@PathVariable String examId, @RequestBody User user, @RequestParam(value = "type", required = true) String engineerType) {
        if (examId.equals(DAILY_QUESTIONS)) {
            return new ResponseEntity<>(examService.randomlyPickQuestionsByEngineerType(DAILY_QUESTIONS_NUM, engineerType, false), HttpStatus.OK);
        }
        if (examId.equals(WEEKLY_QUESTIONS)) {
            return new ResponseEntity<>(examService.randomlyPickQuestionsByEngineerType(0, engineerType, true), HttpStatus.OK);
        }
        Exam exam = examService.getExamById(examId);
        if (exam == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        // 早于考试时间
        if (exam.getStartTime() != null && LocalDateTime.now(CHINA_ZONE).isBefore(exam.getStartTime())) {
            return new ResponseEntity<>(HttpStatus.NOT_ACCEPTABLE);
        }
        List<String> questionList = examService.startExam(exam, user);
        if (questionList == null) {
            return new ResponseEntity<>(HttpStatus.NOT_ACCEPTABLE);
        }
        return new ResponseEntity<>(questionList, HttpStatus.OK);
    }
    @PostMapping("/exams/{examId}/end")
    public ResponseEntity<?> endExam(@PathVariable String examId, @RequestBody ExamHistory examHistory) {
        if (examId.equals(DAILY_QUESTIONS)) {
            // 判断是否已经发送过这个event
            PointsHistory pointsHistory = pointsHistoryService.findLatestPointsHistoryByUserAndEvent(examHistory.getUserId(), Event.DAILY_QUESTIONS);
            if (pointsHistory != null) {
                int today = LocalDateTime.now(CHINA_ZONE).getDayOfYear();
                int lastEventDay = pointsHistory.getCreatedTime().getDayOfYear();
                if (lastEventDay == today) {
                    // 已经加过分了，不再加分。
                    return new ResponseEntity<>(false, HttpStatus.OK);
                }
            }
            boolean isSuccess = eventService.handleEvent(examHistory.getUserId(), Event.DAILY_QUESTIONS);
            return new ResponseEntity<>(isSuccess ? HttpStatus.OK : HttpStatus.NOT_ACCEPTABLE);
        }
        if (examId.equals(WEEKLY_QUESTIONS)) {
            // 判断是否已经发送过这个event
            PointsHistory pointsHistory = pointsHistoryService.findLatestPointsHistoryByUserAndEvent(examHistory.getUserId(), Event.WEEKLY_QUESTION);
            if (pointsHistory != null) {
                int todayWeek = Utils.getWeekNum();
                int lastEventWeek = pointsHistory.getCreatedTime().get(WeekFields.of(Locale.getDefault()).weekOfWeekBasedYear());
                if (lastEventWeek == todayWeek) {
                    // 已经加过分了，不再加分。
                    return new ResponseEntity<>(false, HttpStatus.OK);
                }
            }
            boolean isSuccess = eventService.handleEventWithCustomPoints(examHistory.getUserId(),
                    Event.WEEKLY_QUESTION,
                    examHistory.getPoints(),
                    WEEKLY_POINTS_DESCRIPTION);
            return new ResponseEntity<>(isSuccess ? HttpStatus.OK : HttpStatus.NOT_ACCEPTABLE);
        }
        Exam exam = examService.getExamById(examId);
        if (exam == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        examHistory.setExamId(examId);
        return new ResponseEntity<>(examService.endExam(examHistory) == null ? HttpStatus.CONFLICT : HttpStatus.OK);
    }
    @GetMapping("/exams")
    public ResponseEntity<?> showAllExams(@RequestBody(required = false) User user) {
        if (user == null) {
            return new ResponseEntity<>(examService.showAllExams(), HttpStatus.OK);
        } else {
             return new ResponseEntity<>(examService.showAllAvailableExams(user), HttpStatus.OK);
        }
    }
    @GetMapping("/exams/history")
    public ResponseEntity<?> showExamsHistory(@RequestParam("user") String userName) {
        if (userName == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        } else {
            return new ResponseEntity<>(examService.showExamHistoriesForUser(userName), HttpStatus.OK);
        }
    }
    @GetMapping("/questions/{questionId}")
    public ResponseEntity<?> getQuestion(@PathVariable String questionId) {
        Question q = questionService.findQuestionById(questionId);
        if (q == null) {
           return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(q, HttpStatus.OK);
    }
    @PostMapping("/questions/")
    public ResponseEntity<?> createQuestion(@RequestBody Question question) {
        return new ResponseEntity<>(questionService.createQuestion(question), HttpStatus.OK);
    }
    @PutMapping("/questions/{questionId}")
    public ResponseEntity<?> editQuestion(@PathVariable String questionId, @RequestBody Question updatedQuestion) {
        Question q = questionService.findQuestionById(questionId);
        if (q == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        updatedQuestion.setId(q.getId());
        return new ResponseEntity<>(questionService.editQuestion(updatedQuestion), HttpStatus.OK);
    }
    @DeleteMapping("/questions/{questionId}")
    public ResponseEntity<?> deleteQuestion(@PathVariable String questionId) {
        Question q = questionService.findQuestionById(questionId);
        if (q == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(questionService.deleteQuestion(q), HttpStatus.OK);
    }
    @PostMapping("/exams/{examId}/questions")
    public ResponseEntity<?> addQuestionsToExam(@PathVariable String examId, @RequestBody List<ExamToQuestion> examToQuestions) {
        Exam exam = examService.getExamById(examId);
        if (examService.addQuestionsToExam(exam, examToQuestions)) {
            return new ResponseEntity<>(HttpStatus.OK);
        } else {
            return new ResponseEntity<>(HttpStatus.NOT_ACCEPTABLE);
        }
    }
    @DeleteMapping("/exams/{examId}/questions")
    public ResponseEntity<?> removeQuestionsToExam(@PathVariable String examId, @RequestBody Question question) {
        Exam exam = examService.getExamById(examId);
        if (examService.removeQuestionFromExam(exam, question)) {
            return new ResponseEntity<>(HttpStatus.OK);
        } else {
            return new ResponseEntity<>(HttpStatus.NOT_ACCEPTABLE);
        }
    }
}
